@@ -7,7 +7,7 @@ from collections import deque, namedtuple
 from random import randrange
 
 from beartype import beartype
-from beartype.typing import List, Optional, Callable, Deque
+from beartype.typing import List, Optional, Callable, Deque, Union
 
 import torch
 from torch import nn
@@ -21,6 +21,7 @@ from einops import rearrange, repeat
 from einops.layers.torch import Rearrange
 
 from palm_rlhf_pytorch.palm import PaLM
+from palm_rlhf_pytorch.palm_enc_dec import PaLMEncDec
 from palm_rlhf_pytorch.reward import RewardModel
 from palm_rlhf_pytorch.optimizer import get_optimizer
 from palm_rlhf_pytorch.utils import masked_mean, eval_decorator
@@ -42,8 +43,8 @@ PPOActionCriticReturn = namedtuple('PPOActionCriticReturn', [
 class ActorCritic(nn.Module):
     def __init__(
         self,
-        palm: PaLM,
-        critic_palm: Optional[PaLM] = None,
+        palm: Union[PaLM, PaLMEncDec],
+        critic_palm: Optional[Union[PaLM, PaLMEncDec]] = None,
         pooled_values = False,
         actor_lora = True,
         critic_lora = True,
@@ -62,6 +63,9 @@ class ActorCritic(nn.Module):
         if not exists(self.critic_palm):
             self.critic_palm = copy.deepcopy(palm)
 
+        self.actor_is_enc_dec = isinstance(self.actor_palm, PaLMEncDec)
+        self.critic_is_enc_dec = isinstance(self.critic_palm, PaLMEncDec)
+
         self.actor_palm.set_dropout(actor_dropout)
         self.critic_palm.set_dropout(critic_dropout)
 
@@ -140,8 +144,9 @@ class ActorCritic(nn.Module):
         action_logits, value = self.forward(
             sequence,
             mask = action_mask,
-            return_values = return_values
-        )        
+            return_values = return_values,
+            return_last_n_tokens = action_len
+        )
 
         return PPOActionCriticReturn(
             actions,
@@ -156,18 +161,22 @@ class ActorCritic(nn.Module):
         self,
         x,
         mask = None,
-        return_values = True
+        return_values = True,
+        return_last_n_tokens = None  # if not None, this would return only the last N of the sequence dimension (dimension 2)
     ):
         action_logits = self.actor_palm(
-            x,
+            prompt = x,
             finetune_scope = self.actor_lora_scope
         )
 
+        if exists(return_last_n_tokens):
+            action_logits = action_logits[:, -return_last_n_tokens:]
+
         if not return_values:
             return action_logits, None
 
         critic_embeds = self.critic_palm(
-            x,
+            prompt = x,
             return_only_embedding = True,
             finetune_scope = self.critic_lora_scope
         )
@@ -177,6 +186,9 @@ class ActorCritic(nn.Module):
 
         values = self.value_head(critic_embeds)
 
+        if not self.pooled_values and exists(return_last_n_tokens):
+            values = values[:, -return_last_n_tokens:]
+
         return action_logits, values
 
 # data
@@ -349,6 +361,8 @@ class RLHFTrainer(nn.Module):
 
         self.actor_critic = actor_critic
 
+        assert actor_critic.actor_palm == palm
+
         self.reward_model = reward_model.eval()
 
         # train hyperparameters
@@ -472,17 +486,16 @@ class RLHFTrainer(nn.Module):
                 old_values
             ) in dl:
                 action_masks = ~prompt_masks & masks
+                action_len = old_log_probs.shape[-1]
 
                 action_logits, values = self.actor_critic(
                     sequences,
-                    mask = action_masks
+                    mask = action_masks,
+                    return_last_n_tokens = action_len
                 )
 
-                action_len = old_log_probs.shape[-1]
-
                 action_probs = action_logits.softmax(dim = -1)
                 action_log_probs = log_prob(action_probs, sequences)
-                action_log_probs = action_log_probs[:, -action_len:]
 
                 # calculate entropies, taking into account which part of the sequence is actually an action
 
@@ -500,8 +513,6 @@ class RLHFTrainer(nn.Module):
                 normalize_kwargs = dict()
 
                 if old_values.ndim == 2:
-                    old_values = old_values[:, -action_len:]
-                    values = values[:, -action_len:]
                     rewards = rearrange(rewards, 'b -> b 1')
                     normalize_kwargs = dict(dim = -1, mask = action_masks[:, -action_len:])
 
